@@ -31,7 +31,9 @@
 #' }
 #'
 l_glyph_add_image <- function(widget, images, label="", ...) {
-    return(l_glyph_add.default(widget, "image",
-                       images=images, label=label, ...))
+    structure(
+        l_glyph_add.default(widget, "image", images=images, label=label, ...),
+        class = c("l_glyph_image", "l_glyph", "loon")
+    )
 }
 
