@@ -159,35 +159,43 @@ package ifneeded loon 1.2.0 "
    [list source [file join $dir library/oo_HistogramWorldviewVisual.tcl]]
    [list source [file join $dir library/oo_HistogramLayer.tcl]]
    [list source [file join $dir library/oo_ColorCrossTable.tcl]]
-   [list source [file join $dir library/setMetaKeys.tcl]]
-   [list source [file join $dir library/x_LoonInspector.tcl]]
-   [list source [file join $dir library/applySelectionLogic.tcl]]
-   [list source [file join $dir library/x_baloonHelp.tcl]]
-   [list source [file join $dir library/x_context.tcl]]
-   [list source [file join $dir library/x_resize.tcl]]
-   [list source [file join $dir library/oo_ConditioningScale.tcl]]
-   [list source [file join $dir library/toplevel.tcl]]
-   [list source [file join $dir library/x_synchronize.tcl]]
-   [list source [file join $dir library/data.tcl]]
-   [list source [file join $dir library/image_scale.tcl]]
-   [list source [file join $dir library/loon_dialog.tcl]]
-   [list source [file join $dir library/x_WidgetFactory.tcl]]
-   [list source [file join $dir library/oo_GraphInspectorAnalysis.tcl]]
-   [list source [file join $dir library/oo_TextsWorldviewVisual.tcl]]
-   [list source [file join $dir library/callstack.tcl]]
-   [list source [file join $dir library/x_map.tcl]]
-   [list source [file join $dir library/util-color.tcl]]
-   [list source [file join $dir library/oo_GraphInspectorEdgesAnalysis.tcl]]
-   [list source [file join $dir library/scales_extended.tcl]]
-   [list source [file join $dir library/x_widgets.tcl]]
-   [list source [file join $dir library/oo_GraphInspector.tcl]]
-   [list source [file join $dir library/export.tcl]]
-   [list source [file join $dir library/navgraph.tcl]]
-   [list source [file join $dir library/oo_GraphInspectorNodesAnalysis.tcl]]
-   [list source [file join $dir library/oo_MinMaxScale.tcl]]
-   [list source [file join $dir library/imageviewer.tcl]]
-   [list source [file join $dir library/oo_Serialaxes_Widget.tcl]]
-   [list source [file join $dir library/x_graphutils.tcl]]
-   [list source [file join $dir library/eventsutil.tcl]]
+   [list source [file join $dir library/oo_Rotate3D_Controller.tcl]]
+   [list source [file join $dir library/oo_Scatterplot3D_Model.tcl]]
+   [list source [file join $dir library/oo_Plot3DInspectorAnalysis.tcl]]
+   [list source [file join $dir library/oo_Scatterplot3D_Widget.tcl]]
+   [list source [file join $dir library/oo_Axes3DVisual.tcl]]
+   [list source [file join $dir library/oo_Scatterplot3D_View.tcl]]
+   [list source [file join $dir library/oo_Scatterplot3D_Controller.tcl]]
+   [list source [file join $dir library/oo_Plot3DInspector.tcl]]
    [list source [file join $dir library/oo_TextWorldviewVisual.tcl]]
+   [list source [file join $dir library/oo_GraphInspectorNodesAnalysis.tcl]]
+   [list source [file join $dir library/x_resize.tcl]]
+   [list source [file join $dir library/applySelectionLogic.tcl]]
+   [list source [file join $dir library/x_context.tcl]]
+   [list source [file join $dir library/x_map.tcl]]
+   [list source [file join $dir library/data.tcl]]
+   [list source [file join $dir library/x_synchronize.tcl]]
+   [list source [file join $dir library/setMetaKeys.tcl]]
+   [list source [file join $dir library/oo_GraphInspector.tcl]]
+   [list source [file join $dir library/x_graphutils.tcl]]
+   [list source [file join $dir library/oo_Serialaxes_Widget.tcl]]
+   [list source [file join $dir library/toplevel.tcl]]
+   [list source [file join $dir library/loon_dialog.tcl]]
+   [list source [file join $dir library/scales_extended.tcl]]
+   [list source [file join $dir library/x_WidgetFactory.tcl]]
+   [list source [file join $dir library/x_baloonHelp.tcl]]
+   [list source [file join $dir library/oo_MinMaxScale.tcl]]
+   [list source [file join $dir library/util-color.tcl]]
+   [list source [file join $dir library/oo_ConditioningScale.tcl]]
+   [list source [file join $dir library/export.tcl]]
+   [list source [file join $dir library/eventsutil.tcl]]
+   [list source [file join $dir library/callstack.tcl]]
+   [list source [file join $dir library/navgraph.tcl]]
+   [list source [file join $dir library/imageviewer.tcl]]
+   [list source [file join $dir library/image_scale.tcl]]
+   [list source [file join $dir library/oo_TextsWorldviewVisual.tcl]]
+   [list source [file join $dir library/x_widgets.tcl]]
+   [list source [file join $dir library/oo_GraphInspectorAnalysis.tcl]]
+   [list source [file join $dir library/oo_GraphInspectorEdgesAnalysis.tcl]]
+   [list source [file join $dir library/x_LoonInspector.tcl]]
 "
